//
//  HesabeGatewayVC.m
//  HesabeObjectiveCDemo
//  Copyright © 2020 Hesabe. All rights reserved.
//

#import "HesabeGatewayVC.h"
#import "HesabeCrpyt.h"
#import "Constants.h"

@interface HesabeGatewayVC ()

@end

@implementation HesabeGatewayVC

- (void)viewDidLoad {
    [super viewDidLoad];
    [self checkoutRequest:self.paymentRequest];
}

/// Create request for Payment Token
/// @param paymentRequest : A dictionary of request details
- (void)checkoutRequest:(NSDictionary *)paymentRequest {
    NSError *error;
    NSData *paymentRequestJson = [NSJSONSerialization dataWithJSONObject:paymentRequest options:0 error:&error];
    NSString *paymentRequestEncrypted = [[HesabeCrpyt new] encrypt:paymentRequestJson];
    NSString *postString =[NSString stringWithFormat:@"data=%@",paymentRequestEncrypted];
    NSData *postData = [postString dataUsingEncoding:NSUTF8StringEncoding];
    
    NSMutableURLRequest *urlRequest = [[NSMutableURLRequest alloc] initWithURL:[NSURL URLWithString:CHECKOUT_URL]];
    [urlRequest setHTTPMethod:@"POST"];
    [urlRequest setHTTPBody:postData];
    [urlRequest addValue:ACCESS_CODE forHTTPHeaderField:@"accessCode"];
    
    NSURLSession *session = [NSURLSession sharedSession];
    NSURLSessionDataTask *dataTask = [session dataTaskWithRequest:urlRequest completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
        NSHTTPURLResponse *httpResponse = (NSHTTPURLResponse *)response;
        if(httpResponse.statusCode == 200) {
            NSString *encryptedResponse = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
            if(encryptedResponse != nil) {
                [self checkoutResponse:encryptedResponse];
            } else {
                NSLog(@"Error");
            }
        } else {
            NSLog(@"Error");
        }
    }];
    [dataTask resume];
}

/// Update Payment Token response
/// @param encryptedResponse : The encrypted response received from `CHECKOUT_URL`
- (void)checkoutResponse:(NSString *)encryptedResponse {
    NSData *decryptedResponseData = [[HesabeCrpyt new] decrypt:encryptedResponse];
    NSError* error;
    NSDictionary* responseDictionary = [NSJSONSerialization JSONObjectWithData:decryptedResponseData options:kNilOptions error:&error];
    NSData *responseData = [[responseDictionary valueForKey:@"response"] valueForKey:@"data"];
    NSLog(@"data %@",responseData);
    NSString *token = [NSString stringWithFormat:@"%@",responseData];
    [self redirectToPayment:token];
}

/// loads the PAYMENT_URL with payment token in webView (Payment Gateway)
/// @param token : The payment token extracted from checkoutResponse( )
- (void)redirectToPayment:(NSString *)token {
    NSURL *url = [NSURL URLWithString: [PAYMENT_URL stringByAppendingString:token]];
    NSURLRequest *request = [[NSURLRequest alloc] initWithURL:url];
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.webView loadRequest:request];
    });
}

- (void)loadView {
    [super loadView];
    WKWebViewConfiguration * webConfiguration = [WKWebViewConfiguration new];
    self.webView = [[WKWebView alloc] initWithFrame:CGRectZero configuration:webConfiguration];
    self.webView.navigationDelegate = self;
    self.webView.UIDelegate = self;
    self.view = self.webView;
}

/**
 The gateway URL are handled here. On load of every new url in gateway, the method is being called.
 The `decisionHandler()` allows the method to proceed further or stop processing.
 Here, If the response URL is being called, the process is being stopped to extract the data from that url and proceed to update response.
 */
- (void)webView:(WKWebView *)webView decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler {
    
    NSURL *url = navigationAction.request.URL;
    NSString *responseURLString = [RESPONSE_URL stringByAppendingString:@"/?data"];
    NSString *failureURLString = [FAILURE_URL stringByAppendingString:@"/?data"];
    if (url != nil && ([url.absoluteString containsString:responseURLString] || [url.absoluteString containsString:failureURLString])) {
        decisionHandler(WKNavigationActionPolicyCancel);
        NSArray *urlComponents = [url.absoluteString componentsSeparatedByString:@"="];
        NSString *encryptedResponse = urlComponents[1];
        [self paymentResponse:encryptedResponse];
    } else {
        decisionHandler(WKNavigationActionPolicyAllow);
    }
}

/// Updates Payment response post completion
/// @param encryptedResponse : The encrypted response received on `RESPONSE_URL` or  `FAILURE_URL`
- (void)paymentResponse:(NSString *)encryptedResponse {
    NSData *decryptedResponseData = [[HesabeCrpyt new] decrypt:encryptedResponse];
    NSError* error;
    NSDictionary* responseDictionary = [NSJSONSerialization JSONObjectWithData:decryptedResponseData options:kNilOptions error:&error];
    if (error) {
        NSString *decryptedResponseString = [[NSString alloc] initWithData:decryptedResponseData encoding:NSUTF8StringEncoding];
        NSRange range = [decryptedResponseString rangeOfString:@"}}"];
        if (range.location != NSNotFound) {
            decryptedResponseString = [[decryptedResponseString substringToIndex:range.location] stringByAppendingFormat:@"}}"];
        }
        decryptedResponseData = [decryptedResponseString dataUsingEncoding:NSUTF8StringEncoding];
        responseDictionary = [NSJSONSerialization JSONObjectWithData:decryptedResponseData options:kNilOptions error:&error];
    }
    NSDictionary *responseData = [responseDictionary valueForKey:@"response"];
    PaymentResponse *paymentResponse = [[PaymentResponse alloc] initWithDictionary:responseData];
    [self dismissViewControllerAnimated:true completion:^{
        [self.delegate paymentResponse:paymentResponse];
    }];
}

@end
