//
//  HesabeCrpyt.m
//  HesabeObjectiveCDemo
//  Copyright © 2020 Hesabe. All rights reserved.
//

#import "HesabeCrpyt.h"
#import "Constants.h"
//#import <CommonCrypto/CommonCryptor.h>

@implementation HesabeCrpyt

@synthesize key = _key;
@synthesize iv = _iv;

- (instancetype)init
{
    self = [super init];
    if (self) {
        _key = [MERCHANT_KEY dataUsingEncoding:NSUTF8StringEncoding];
        _iv = [MERCHANT_IV dataUsingEncoding:NSUTF8StringEncoding];
    }
    return self;
}

- (NSString *)encrypt:(NSData *)data {
    NSData *paymentRequestEncrypted = [self crypt:data context:kCCEncrypt];
    return [self hexString:paymentRequestEncrypted];
}

- (NSData *)decrypt:(NSString *)hexString {
    NSData *encryptedData = [self dataFromHexString: hexString];
    return [self crypt:encryptedData context:kCCDecrypt];
}

- (NSData *)crypt:(NSData *)data context:(CCOperation)encryptOrDecrypt
{
    NSInteger cryptLength = data.length + kCCBlockSizeAES128;
    NSMutableData  *cryptData = [NSMutableData dataWithLength:cryptLength];
    
    NSInteger keyLength = _key.length;
    CCOptions options = kCCOptionPKCS7Padding;
    size_t bytesLength = 0;
    
    CCCryptorStatus ccStatus = CCCrypt(encryptOrDecrypt, kCCAlgorithmAES, options, _key.bytes, keyLength, _iv.bytes, data.bytes, data.length, cryptData.mutableBytes, cryptLength, &bytesLength);
    
    if (ccStatus != kCCSuccess) {
        NSLog(@"CCCrypt status: %d", ccStatus);
    }
    
    cryptData.length = bytesLength;
    
    return cryptData;
}

- (NSString *)hexString:(NSData *) data {
    const unsigned char *bytes = (const unsigned char *) data.bytes;
    NSMutableString *hex = [NSMutableString new];
    for (NSInteger i = 0; i < data.length; i++) {
        [hex appendFormat:@"%02x", bytes[i]];
    }
    return [hex copy];
}

- (NSData *)dataFromHexString:(NSString *)string
{
    string = [string lowercaseString];
    NSMutableData *data= [NSMutableData new];
    unsigned char whole_byte;
    char byte_chars[3] = {'\0','\0','\0'};
    int i = 0;
    NSInteger length = string.length;
    while (i < length-1) {
        char c = [string characterAtIndex:i++];
        if (c < '0' || (c > '9' && c < 'a') || c > 'f')
            continue;
        byte_chars[0] = c;
        byte_chars[1] = [string characterAtIndex:i++];
        whole_byte = strtol(byte_chars, NULL, 16);
        [data appendBytes:&whole_byte length:1];
    }
    return data;
}

@end
